/////////////////////////////////////////////////////////////////////
// MT4-Bobokus-Fibos.mq4
// Version 1.0
// Copyright  2008, Bruce Hellstrom (brucehvn)
// bhweb@speakeasy.net
// http://www.metaquotes.net
/////////////////////////////////////////////////////////////////////
#property copyright "Copyright  2008, Bruce Hellstrom (brucehvn)"
#property link      "http://www.metaquotes.net"
#property show_inputs

/////////////////////////////////////////////////////////////////////
// Description
/////////////////////////////////////////////////////////////////////
/*

This script will draw a set of Bobokus fibs on a chart and then exit. The user can specify multiple parameters for the fib set including color, name, timeframe, etc. It is expected that this will be used to put the fibs on the chart, whereby then the user can adjust them to the high/low points as desired. This script just keeps people from having to modify the fib names, etc. each time.

*/ 

/////////////////////////////////////////////////////////////////////
// User Inputs
/////////////////////////////////////////////////////////////////////
/*

ShowRS - Flag to indicate whether to show the R1/S1 levels. Swing fibs and retracement fibs do not require these levels.

ShowAsRay - Flag to indicate whether the "Ray" setting is set on the fib set. The Ray setting if on makes the fibs extend all the way to the right side of the screen. This can be toggled later via the fib properties.

ShowAboveH4 - Flag to indicate whether this set of fibs should be shown on timeframes greater than H4. This is helpful to reduce clutter when viewing daily, weekly, monthly charts. Typically, intraday and swing fibs should have this set to false. 

Fibo_Num - Integer value to help uniquely identify this set of fibs. Fib object names are derived from Fibo_Num, Fibo_Name, and TimeFrame. This number generally doesn't need to be changed unless you have 2 sets of a particular fib like Intraday. 

Fibo_Name - String that describes this fib set. This will be displayed on each level of the fib. It is also used to make the unique name of the fib set in the objects list. Fib object names are derived from Fibo_Num, Fibo_Name, and TimeFrame. 

TimeFrame - String value to indicate where starting values should be calculated. Valid values are "H1", "H4", "D1", "W1", and "MN1". The script will look to the previous period specified for high/low values to set the starting point of the fib set. It is expected that the user will move the fib set once shown on the chart. 
            
Fibo_color - The color to draw the fib set.


Cust_Time - This variable indicates the date/time that should be used as a starting point for the fib set. If not changed, then the current date/time is used. This setting is useful for testing in the strategy tester to put the fibs on a particular date.

The format of this field is: "yyyy.mm.dd hh:mm"

For example, to set the starting date to Sep 24, 2007 at 00:00 server time, then set this field to:

2007.09.24 00:00
                
*/                


// User inputs
extern bool ShowRS = true;  // don't need S1 and R1 for swing/retracement fibs
extern bool ShowAsRay = true;  // Set the ray flag on the fibos
extern bool ShowAboveH4 = true; // Sets visibility to all timeframes, false means don't show on daily and above
extern int Fibo_Num = 12345;
extern string Fibo_Name = "Intraday";
extern string TimeFrame = "D1";
extern color Fibo_color = DodgerBlue;
extern datetime Cust_Time = 0;

double HiPrice, LoPrice;
datetime StartTime;
int Tfmin = PERIOD_H1;
string Object_Name = "";


/////////////////////////////////////////////////////////////////////
// Script initialization function
/////////////////////////////////////////////////////////////////////

int init() {
    Object_Name = Fibo_Name + Fibo_Num + "_" + TimeFrame;

    if ( TimeFrame == "H1" ) {
        Tfmin = PERIOD_H1;
    }
    else if ( TimeFrame == "H4" ) {
        Tfmin = PERIOD_H4;
    }
    else if ( TimeFrame == "D1" ) {
        Tfmin = PERIOD_D1;
    }
    else if ( TimeFrame == "W1" ) {
        Tfmin = PERIOD_W1;
    }
    else if ( TimeFrame == "MN1" ) {
        Tfmin = PERIOD_MN1;
    }
    else {
        Tfmin = -1;
    }
    
    if ( Tfmin == -1 ) {
        Print( "Invalid time frame selected" );
        return( -1 );
    }
    return( 0 );
}

/////////////////////////////////////////////////////////////////////
// Custom indicator deinitialization function
/////////////////////////////////////////////////////////////////////

int deinit() {
    return( 0 );
}

/////////////////////////////////////////////////////////////////////
// Indicator Logic run on every tick
/////////////////////////////////////////////////////////////////////

int start() {
    if ( ObjectFind( Object_Name ) != -1 ) {
        Print( "Fibos already drawn with that name/unique number." );
        return( -1 );
    }
    
	if ( Cust_Time == 0 ) {
        StartTime	= iTime(NULL, Tfmin, 0);
    }
    else {
        StartTime = Cust_Time;
    }
	int shift	= iBarShift( NULL, Tfmin, StartTime ) + 1;	// yesterday
	HiPrice		= iHigh(NULL, Tfmin, shift);
	LoPrice		= iLow (NULL, Tfmin, shift);

	if( TimeDayOfWeek( StartTime ) == 0/*Sunday*/)
	{
		HiPrice = MathMax( HiPrice, iHigh( NULL, Tfmin, shift + 1 ) );
		LoPrice = MathMin( LoPrice, iLow( NULL, Tfmin, shift + 1 ) );
	}
    
    //Print( "Tfmin: ", Tfmin, " StartTime: ", TimeToStr( StartTime ), " shift: ", shift, " HiPrice: ", HiPrice, " LoPrice: ", LoPrice );

	DrawFibo();
	
    return( -1 );
}

/////////////////////////////////////////////////////////////////////
// DrawFibo()
/////////////////////////////////////////////////////////////////////

int DrawFibo()
{
	if ( ObjectFind( Object_Name ) == -1 ) {
		ObjectCreate( Object_Name, OBJ_FIBO, 0, StartTime, HiPrice, StartTime, LoPrice );

        ObjectSet( Object_Name, OBJPROP_LEVELCOLOR, Fibo_color );
        
        int fiblevels = 11;
        int fibctr = 0;
        
        if ( !ShowRS ) {
            fiblevels = 9;
        }
        
        ObjectSet( Object_Name, OBJPROP_FIBOLEVELS, fiblevels );
        ObjectSet( Object_Name, OBJPROP_FIRSTLEVEL + fibctr, -0.55 );
        ObjectSetFiboDescription( Object_Name, fibctr, Fibo_Name + " Target 2 - %$");
        fibctr++;
        
        ObjectSet( Object_Name, OBJPROP_FIRSTLEVEL + fibctr, -0.34 );
        ObjectSetFiboDescription( Object_Name, fibctr, Fibo_Name + " Target 1 - %$");
        fibctr++;
        
        ObjectSet( Object_Name, OBJPROP_FIRSTLEVEL + fibctr, 0.0 );
        ObjectSetFiboDescription( Object_Name, fibctr, Fibo_Name + " Low - %$");
        fibctr++;
        
        if ( ShowRS ) {
            ObjectSet( Object_Name, OBJPROP_FIRSTLEVEL + fibctr, 0.191 );
            ObjectSetFiboDescription( Object_Name, fibctr, Fibo_Name + " S1 - %$");
            fibctr++;
        }
        ObjectSet( Object_Name, OBJPROP_FIRSTLEVEL + fibctr, 0.382 );
        ObjectSetFiboDescription( Object_Name, fibctr, Fibo_Name + " Short - %$");
        fibctr++;
        
        ObjectSet( Object_Name, OBJPROP_FIRSTLEVEL + fibctr, 0.50 );
        ObjectSetFiboDescription( Object_Name, fibctr, Fibo_Name + " Pivot - %$");
        fibctr++;
        
        ObjectSet( Object_Name, OBJPROP_FIRSTLEVEL + fibctr, 0.618 );
        ObjectSetFiboDescription( Object_Name, fibctr, Fibo_Name + " Long - %$");
        fibctr++;
        
        if ( ShowRS ) {
            ObjectSet( Object_Name, OBJPROP_FIRSTLEVEL + fibctr, 0.809 );
            ObjectSetFiboDescription( Object_Name, fibctr, Fibo_Name + " R1 - %$");
            fibctr++;
        }
        ObjectSet( Object_Name, OBJPROP_FIRSTLEVEL + fibctr, 1.0 );
        ObjectSetFiboDescription( Object_Name, fibctr, Fibo_Name + " High - %$");
        fibctr++;
        
        ObjectSet( Object_Name, OBJPROP_FIRSTLEVEL + fibctr, 1.34 );
        ObjectSetFiboDescription( Object_Name, fibctr, Fibo_Name + " Target 1 - %$");
        fibctr++;
        
        ObjectSet( Object_Name, OBJPROP_FIRSTLEVEL + fibctr, 1.55 );
        ObjectSetFiboDescription( Object_Name, fibctr, Fibo_Name + " Target 2 - %$");
    
        ObjectSet( Object_Name, OBJPROP_RAY, ShowAsRay );
        ObjectSet( Object_Name, OBJPROP_BACK, true);
        
        if ( !ShowAboveH4 ) {
            int Tfs = OBJ_PERIOD_M1 | OBJ_PERIOD_M5 | OBJ_PERIOD_M15 | OBJ_PERIOD_M30 |
                      OBJ_PERIOD_H1 | OBJ_PERIOD_H4;
            ObjectSet( Object_Name, OBJPROP_TIMEFRAMES, Tfs );
        }
    }
}
