//+---------------------------------------------------------------------------
//| Stochastic Slope.mq4                                                      
//| Copyright  2010, Xaphod                                                  
//| http://forexwhiz.appspot.com                                          
//| Description: Indicator that shows the slope stochastic main line is above  
//|              or below a set value by coloring it.   
//|              Since we have different units on the x and y axis, the term  
//|              slope is used here instead of angle.                         
//|              The threshold value for the slope is a value between 0.0 and
//|              1.0. As the slope you see varies with the height of the chart, 
//|              you will have to empirically decide what value works for you.
//| Change Log:                                                               
//|   2010-03-22, Xaphod - Implementation. v1.0 Release
//|   2010-03-25, Xaphod - Added indicator line width properties. v1.01
//+---------------------------------------------------------------------------
#property copyright "Copyright  2010, Xaphod"
#property link      "http://forexwhiz.appspot.com"

//---- Indicator properties
#property indicator_separate_window
#property indicator_minimum 0
#property indicator_maximum 100
#property indicator_buffers 4
#property indicator_color1  Red
#property indicator_color2  DarkSeaGreen
#property indicator_color3  DarkSeaGreen
#property indicator_color4  Blue
#property indicator_level1  20.0
#property indicator_level2  80.0
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2
#property indicator_width4  1
#property indicator_style4  STYLE_DOT

//---- Indicator input parameters
extern string    SettingsInfo="<< %K Period, %D Period, Slowing >>";
extern int       KPeriod=8;
extern int       DPeriod=3;
extern int       Slowing=3;
extern string    PriceInfo="<< 0=High/Low, 1=Close/Close >>";
extern int       PriceField=1;
extern string    MAInfo="<< SMA=0, EMA=1, SMMA=2, LWMA=3 >>";
extern int       MAMethod=0;
extern string    SlopeThreshholdInfo="<< Slope threshold=0.0 to 1.0 >>";
extern double    SlopeThreshold=0.05;


//---- Indicator buffers
double dExtMapBuffer1[];
double dExtMapBuffer2[];
double dExtMapBuffer3[];
double dExtMapBuffer4[];

//---- Indicator name & version
string sIndicatorName;
string sVersion = "v1.01";

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init() {
 
  //---- Setup Indicators   
  SetIndexStyle(0,DRAW_LINE,STYLE_SOLID);
  SetIndexBuffer(0,dExtMapBuffer1);
  SetIndexStyle(1,DRAW_LINE,STYLE_SOLID);
  SetIndexBuffer(1,dExtMapBuffer2);
  SetIndexStyle(2,DRAW_LINE,STYLE_SOLID);
  SetIndexBuffer(2,dExtMapBuffer3);   
  SetIndexStyle(3,DRAW_LINE,STYLE_DOT);
  SetIndexBuffer(3,dExtMapBuffer4);
    
  //---- Set indicator name  
  sIndicatorName = "[Stoch(" + KPeriod + "," + DPeriod + "," + Slowing+") " + "with Slope " + sVersion + "]";
  IndicatorShortName(sIndicatorName);

  // Validate parameters
  if (SlopeThreshold>1) SlopeThreshold=1;
  if (SlopeThreshold<0) SlopeThreshold=0;

  return(0);
}


//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit() {
  //----   
  return(0);
}


//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start() {
  double dStocMain0;    // Value Main (%K Period) on 0 bar
  double dStocMain1;    // Value Main (%K Period)  on 1 bar
  double dStocSignal0;  // Value Signal (%D Period)  on 0 bar 
  double dSlope;        // Slope from 1 bar to 0 bar
  int    iNewBars;      // New bars to be processed
  int    iCountedBars=IndicatorCounted();
  
  // Get new bars
  if(iCountedBars<0) return(-1);  
  if(iCountedBars>0) iCountedBars--;
  iNewBars=Bars-iCountedBars;
  
  //---- main loop
  for(int i=iNewBars-2; i>=0; i--) {
    // Calc stochastic values and slope
    dStocMain0 = iStochastic(NULL,0,8,3,3,MODE_SMA,0,MODE_MAIN,  i); // 0 bar
    dStocMain1 = iStochastic(NULL,0,8,3,3,MODE_SMA,0,MODE_MAIN,  i+1); // 1st bar
    dStocSignal0 = iStochastic(NULL,0,8,3,3,MODE_SMA,0,MODE_SIGNAL,i);// 0 bar
    dSlope=((dStocMain0-dStocMain1)/100);
          
    // Set/reset indicator buffers      
    dExtMapBuffer1[i]=dStocMain0;  // Draw the stoch main (%K) line
    dExtMapBuffer4[i]=dStocSignal0;// Draw the stoch signal (%D) line
    dExtMapBuffer2[i]=EMPTY_VALUE; // Clear the buffers used for the 'slope below the threshold' curve
    dExtMapBuffer3[i]=EMPTY_VALUE;          
    
    // Draw the part of the charts with slope < threshold on top of the main curve
    // Two buffers needed because of how MT4 handles drawing/coloring indicators
    if (dSlope>SlopeThreshold || dSlope<-SlopeThreshold) {
      if (dExtMapBuffer2[i+1]!=EMPTY_VALUE) {
        dExtMapBuffer2[i]=dStocMain0;
      }
      else if (dExtMapBuffer2[i+2]==EMPTY_VALUE) {
        dExtMapBuffer2[i]=dStocMain0;
        dExtMapBuffer2[i+1]=dStocMain1;
      }
      else {
        dExtMapBuffer3[i]=dStocMain0;
        dExtMapBuffer3[i+1]=dStocMain1;
      }    
    }  //end if
  } //end for
  
  return(0);
}
//+------------------------------------------------------------------+